@@ -20,7 +20,7 @@ package MongoDB::Collection;
 # ABSTRACT: A MongoDB Collection
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use MongoDB::Error;
 use MongoDB::IndexView;
@@ -31,16 +31,21 @@ use MongoDB::_Query;
 use MongoDB::Op::_Aggregate;
 use MongoDB::Op::_BatchInsert;
 use MongoDB::Op::_BulkWrite;
+use MongoDB::Op::_Count;
 use MongoDB::Op::_CreateIndexes;
 use MongoDB::Op::_Delete;
 use MongoDB::Op::_Distinct;
+use MongoDB::Op::_FindAndDelete;
+use MongoDB::Op::_FindAndUpdate;
 use MongoDB::Op::_InsertOne;
 use MongoDB::Op::_ListIndexes;
+use MongoDB::Op::_ParallelScan;
 use MongoDB::Op::_Update;
 use MongoDB::_Types qw(
     BSONCodec
     NonNegNum
     ReadPreference
+    ReadConcern
     WriteConcern
 );
 use Types::Standard qw(
@@ -117,6 +122,23 @@ has write_concern => (
     coerce   => WriteConcern->coercion,
 );
 
+#pod =attr read_concern
+#pod
+#pod A L<MongoDB::ReadConcern> object.  May be initialized with a hash
+#pod reference or a string that will be coerced into the level of read
+#pod concern.
+#pod
+#pod By default it will be inherited from a L<MongoDB::Database> object.
+#pod
+#pod =cut
+
+has read_concern => (
+    is       => 'ro',
+    isa      => ReadConcern,
+    required => 1,
+    coerce   => ReadConcern->coercion,
+);
+
 #pod =attr max_time_ms
 #pod
 #pod Specifies the default maximum amount of time in milliseconds that the
@@ -241,6 +263,7 @@ sub _build__op_args {
         bson_codec      => $self->bson_codec,
         coll_name       => $self->name,
         write_concern   => $self->write_concern,
+        read_concern    => $self->read_concern,
         read_preference => $self->read_preference,
         full_name       => join( ".", $self->database->name, $self->name ),
     };
@@ -312,6 +335,7 @@ sub with_codec {
 #pod =method insert_one
 #pod
 #pod     $res = $coll->insert_one( $document );
+#pod     $res = $coll->insert_one( $document, $options );
 #pod     $id = $res->inserted_id;
 #pod
 #pod Inserts a single L<document|/Document> into the database and returns a
@@ -321,6 +345,14 @@ sub with_codec {
 #pod serialized for the database without modifying the original document.
 #pod The generated C<_id> may be retrieved from the result object.
 #pod
+#pod An optional hash reference of options may be given.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod
 #pod =cut
 
 # args not unpacked for efficiency; args are self, document
@@ -330,6 +362,7 @@ sub insert_one {
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_InsertOne->_new(
+            ( defined $_[2] ? (%{$_[2]}) : () ),
             document => $_[1],
             %{ $_[0]->_op_args },
         )
@@ -350,11 +383,17 @@ sub insert_one {
 #pod serialized for the database without modifying the original document.
 #pod The generated C<_id> may be retrieved from the result object.
 #pod
-#pod An optional hash reference of options may be provided.  The only valid option
-#pod is C<ordered>, which defaults to true.  When true, the server will halt
-#pod insertions after the first error (if any).  When false, all documents will be
-#pod processed and any error will only be thrown after all insertions are
-#pod attempted.
+#pod An optional hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<ordered> – when true, the server will halt insertions after the first
+#pod   error (if any).  When false, all documents will be processed and any
+#pod   error will only be thrown after all insertions are attempted.  The
+#pod   default is true.
 #pod
 #pod On MongoDB servers before version 2.6, C<insert_many> bulk operations are
 #pod emulated with individual inserts to capture error information.  On 2.6 or
@@ -368,13 +407,16 @@ sub insert_many {
     MongoDB::UsageError->throw("documents argument must be an array reference")
       unless ref( $_[1] ) eq 'ARRAY';
 
-    # ordered defaults to true
     return MongoDB::InsertManyResult->_new(
         acknowledged => $_[0]->write_concern->is_acknowledged,
         inserted     => $_[0]->client->send_write_op(
             MongoDB::Op::_BulkWrite->_new(
+                # default
+                ordered => 1,
+                # user overrides
+                ( defined $_[2] ? ( %{ $_[2] } ) : () ),
+                # un-overridable
                 queue => [ map { [ insert => $_ ] } @{ $_[1] } ],
-                ordered => ( ( defined $_[2] && exists $_[2]->{ordered} ) ? $_[2]->{ordered} : 1 ),
                 %{ $_[0]->_op_args },
             )
           )->inserted,
@@ -444,9 +486,15 @@ sub delete_many {
 #pod The replacement document must not have any field-update operators in it (e.g.
 #pod C<$set>).
 #pod
-#pod A hash reference of options may be provided.  The only valid option is
-#pod C<upsert>, which defaults to false.  If provided and true, the replacement
-#pod document will be upserted if no matching document exists.
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<upsert> – defaults to false; if true, a new document will be added if one
+#pod   is not found
 #pod
 #pod =cut
 
@@ -457,10 +505,10 @@ sub replace_one {
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_Update->_new(
+            ( defined $_[3] ? (%{$_[3]}) : () ),
             filter     => $_[1],
             update     => $_[2],
             multi      => false,
-            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
             is_replace => 1,
             %{ $_[0]->_op_args },
         )
@@ -479,10 +527,16 @@ sub replace_one {
 #pod The update document must have only field-update operators in it (e.g.
 #pod C<$set>).
 #pod
-#pod A hash reference of options may be provided.  The only valid option is
-#pod C<upsert>, which defaults to false.  If provided and true, a new document will
-#pod be inserted by taking the filter expression and applying the update document
-#pod operations to it prior to insertion.
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<upsert> – defaults to false; if true, a new document will be added if
+#pod   one is not found by taking the filter expression and applying the update
+#pod   document operations to it prior to insertion.
 #pod
 #pod =cut
 
@@ -493,10 +547,10 @@ sub update_one {
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_Update->_new(
+            ( defined $_[3] ? (%{$_[3]}) : () ),
             filter     => $_[1],
             update     => $_[2],
             multi      => false,
-            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
             is_replace => 0,
             %{ $_[0]->_op_args },
         )
@@ -515,10 +569,16 @@ sub update_one {
 #pod The update document must have only field-update operators in it (e.g.
 #pod C<$set>).
 #pod
-#pod A hash reference of options may be provided.  The only valid option is
-#pod C<upsert>, which defaults to false.  If provided and true, a new document will
-#pod be inserted by taking the filter document and applying the update document
-#pod operations to it prior to insertion.
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<upsert> – defaults to false; if true, a new document will be added if
+#pod   one is not found by taking the filter expression and applying the update
+#pod   document operations to it prior to insertion.
 #pod
 #pod =cut
 
@@ -529,10 +589,10 @@ sub update_many {
 
     return $_[0]->client->send_write_op(
         MongoDB::Op::_Update->_new(
+            ( defined $_[3] ? (%{$_[3]}) : () ),
             filter     => $_[1],
             update     => $_[2],
             multi      => true,
-            upsert     => $_[3] && $_[3]->{upsert} ? true : false,
             is_replace => 0,
             %{ $_[0]->_op_args },
         )
@@ -564,6 +624,10 @@ sub update_many {
 #pod   string values: C<'non_tailable'> (the default), C<'tailable'>, and
 #pod   C<'tailable_await'>.
 #pod * C<limit> – the maximum number of documents to return.
+#pod * C<maxAwaitTimeMS> – the maximum amount of time for the server to wait on
+#pod   new documents to satisfy a tailable cursor query. This only applies
+#pod   to a C<cursorType> of 'tailable_await'; the option is otherwise ignored.
+#pod   (Note, this will be ignored for servers before version 3.2.)
 #pod * C<maxTimeMS> – the maximum amount of time to allow the query to run. If
 #pod   C<$maxTimeMS> also exists in the modifiers document, the C<maxTimeMS> field
 #pod   overwrites C<$maxTimeMS>. (Note, this will be ignored for servers before
@@ -624,6 +688,7 @@ sub find {
             comment             => '',
             cursorType          => 'non_tailable',
             limit               => 0,
+            maxAwaitTimeMS      => 0,
             maxTimeMS           => 0,
             noCursorTimeout     => 0,
             oplogReplay         => 0,
@@ -691,6 +756,7 @@ sub find_one {
         comment             => '',
         cursorType          => 'non_tailable',
         limit               => 0,
+        maxAwaitTimeMS      => 0,
         maxTimeMS           => 0,
         noCursorTimeout     => 0,
         oplogReplay         => 0,
@@ -704,6 +770,28 @@ sub find_one {
     )->execute->next;
 }
 
+#pod =method find_id
+#pod
+#pod     $doc = $collection->find_id( $id );
+#pod     $doc = $collection->find_id( $id, $projection );
+#pod     $doc = $collection->find_id( $id, $projection, $options );
+#pod
+#pod Executes a query with a L<filter expression|/Filter expression> of C<< { _id
+#pod => $id } >> and returns a single document.
+#pod
+#pod See the L<find_one|/find_one> documentation for details on the $projection and $options parameters.
+#pod
+#pod See also core documentation on querying:
+#pod L<http://docs.mongodb.org/manual/core/read/>.
+#pod
+#pod =cut
+
+sub find_id {
+    my $self = shift;
+    my $id = shift;
+    return $self->find_one({ _id => $id }, @_);
+}
+
 #pod =method find_one_and_delete
 #pod
 #pod     $doc = $coll->find_one_and_delete( $filter );
@@ -732,6 +820,7 @@ sub find_one_and_delete {
       unless ref( $_[1] );
 
     my ( $self, $filter, $options ) = @_;
+    $options ||= {};
 
     # rename projection -> fields
     $options->{fields} = delete $options->{projection} if exists $options->{projection};
@@ -744,14 +833,13 @@ sub find_one_and_delete {
     # coerce to IxHash
     __ixhash($options, 'sort');
 
-    my @command = (
-        findAndModify => $self->name,
-        query         => $filter,
-        remove        => true,
-        %$options,
+    my $op = MongoDB::Op::_FindAndDelete->_new(
+        %{ $_[0]->_op_args },
+        filter        => $filter,
+        options       => $options,
     );
 
-    return $self->_try_find_and_modify( \@command );
+    return $self->client->send_write_op($op);
 }
 
 #pod =method find_one_and_replace
@@ -769,6 +857,8 @@ sub find_one_and_delete {
 #pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.
 #pod * C<projection> - a hash reference defining fields to return. See "L<limit
@@ -809,6 +899,8 @@ sub find_one_and_replace {
 #pod A hash reference of options may be provided. Valid keys include:
 #pod
 #pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
 #pod * C<projection> - a hash reference defining fields to return. See "L<limit
@@ -858,6 +950,8 @@ sub find_one_and_update {
 #pod =for :list
 #pod * C<allowDiskUse> – if, true enables writing to temporary files.
 #pod * C<batchSize> – the number of documents to return per batch.
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled.
+#pod   (Note, this will be ignored for servers before version 3.2.)
 #pod * C<explain> – if true, return a single document with execution information.
 #pod * C<maxTimeMS> – the maximum amount of time in milliseconds to allow the
 #pod   command to run.  (Note, this will be ignored for servers before version 2.6.)
@@ -909,6 +1003,7 @@ sub aggregate {
         pipeline   => $pipeline,
         options    => $options,
         ( $read_pref ? ( read_preference => $read_pref ) : () ),
+        read_concern => $self->read_concern,
     );
 
     return $self->client->send_read_op($op);
@@ -953,9 +1048,13 @@ sub count {
     # string is OK so we check ref, not just exists
     __ixhash($options, 'hint') if ref $options->{hint};
 
-    my $res = $self->_run_command(
-        Tie::IxHash->new( count => $self->name, query => $filter, %$options ),
-        $self->read_preference );
+    my $op = MongoDB::Op::_Count->_new(
+        options         => $options,
+        filter          => $filter,
+        %{ $self->_op_args },
+    );
+
+    my $res = $self->client->send_read_op($op);
 
     return $res->{n};
 }
@@ -1008,6 +1107,7 @@ sub distinct {
         filter          => $filter,
         options         => $options,
         read_preference => $self->read_preference,
+        read_concern    => $self->read_concern,
     );
 
     return $self->client->send_read_op($op);
@@ -1041,14 +1141,9 @@ sub parallel_scan {
 
     my $db   = $self->database;
 
-    my @command = ( parallelCollectionScan => $self->name, numCursors => $num_cursors );
-
-    my $op = MongoDB::Op::_Command->_new(
-        db_name         => $db->name,
-        query           => \@command,
-        query_flags     => {},
-        bson_codec      => $self->bson_codec,
-        read_preference => $self->read_preference,
+    my $op = MongoDB::Op::_ParallelScan->_new(
+        %{ $self->_op_args },
+        num_cursors     => $num_cursors,
     );
 
     my $result = $self->client->send_read_op( $op );
@@ -1068,7 +1163,7 @@ sub parallel_scan {
             _batch_size   => scalar @$batch,
             _cursor_at    => 0,
             _limit        => 0,
-            _cursor_id    => MongoDB::QueryResult::_pack_cursor_id($c->{id}),
+            _cursor_id    => $c->{id},
             _cursor_start => 0,
             _cursor_flags => {},
             _cursor_num   => scalar @$batch,
@@ -1142,11 +1237,20 @@ sub drop {
 #pod
 #pod The method C<initialize_ordered_bulk_op> may be used as an alias.
 #pod
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod
 #pod =cut
 
 sub initialize_ordered_bulk_op {
-    my ($self) = @_;
-    return MongoDB::BulkWrite->new( collection => $self, ordered => 1 );
+    my ($self, $args) = @_;
+    $args ||= {};
+    return MongoDB::BulkWrite->new( %$args, collection => $self, ordered => 1, );
 }
 
 #pod =method unordered_bulk
@@ -1157,11 +1261,20 @@ sub initialize_ordered_bulk_op {
 #pod
 #pod The method C<initialize_unordered_bulk_op> may be used as an alias.
 #pod
+#pod A hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod
 #pod =cut
 
 sub initialize_unordered_bulk_op {
-    my ($self) = @_;
-    return MongoDB::BulkWrite->new( collection => $self, ordered => 0 );
+    my ($self, $args) = @_;
+    $args ||= {};
+    return MongoDB::BulkWrite->new( %$args, collection => $self, ordered => 0 );
 }
 
 #pod =method bulk_write
@@ -1207,10 +1320,16 @@ sub initialize_unordered_bulk_op {
 #pod Valid method names include C<insert_one>, C<insert_many>, C<delete_one>,
 #pod C<delete_many> C<replace_one>, C<update_one>, C<update_many>.
 #pod
-#pod An optional hash reference of options may be provided.  The only valid value
-#pod is C<ordered>. It defaults to true.  When true, the bulk operation is executed
-#pod like L</initialize_ordered_bulk>. When false, the bulk operation is executed
-#pod like L</initialize_unordered_bulk>.
+#pod An optional hash reference of options may be provided.
+#pod
+#pod Valid options include:
+#pod
+#pod =for :list
+#pod * C<bypassDocumentValidation> - skips document validation, if enabled; this
+#pod   is ignored for MongoDB servers older than version 3.2.
+#pod * C<ordered> – when true, the bulk operation is executed like
+#pod   L</initialize_ordered_bulk>. When false, the bulk operation is executed
+#pod   like L</initialize_unordered_bulk>.  The default is true.
 #pod
 #pod See L<MongoDB::BulkWrite> for more details on bulk writes.  Be advised that
 #pod the legacy Bulk API method names differ slightly from MongoDB::Collection
@@ -1227,9 +1346,12 @@ sub bulk_write {
     MongoDB::UsageError->throw("options not a hash reference")
       if defined($options) && ref($options) ne 'HASH';
 
-    $options ||= { ordered => 1 };
+    $options ||= {};
 
-    my $bulk = $options->{ordered} ? $self->ordered_bulk : $self->unordered_bulk;
+    my $ordered = exists $options->{ordered} ? delete $options->{ordered} : 1;
+
+    my $bulk =
+      $ordered ? $self->ordered_bulk($options) : $self->unordered_bulk($options);
 
     my $i = 0;
 
@@ -1262,7 +1384,7 @@ sub bulk_write {
             $bulk->insert_one($_) for @$args;
         }
         else {
-            my ($filter, $doc, $options) = @$args;
+            my ($filter, $doc, $opts) = @$args;
 
             my $view = $bulk->find($filter);
 
@@ -1277,7 +1399,7 @@ sub bulk_write {
             }
 
             # updates might be upserts
-            $view = $view->upsert if $options && $options->{upsert};
+            $view = $view->upsert if $opts && $opts->{upsert};
 
             # handle updates
             if ( $method eq 'replace_one' ) {
@@ -1342,14 +1464,18 @@ sub _find_one_and_update_or_replace {
         $options->{new} = delete( $options->{returnDocument} ) eq 'after' ? true : false;
     }
 
-    my @command = (
-        findAndModify => $self->name,
-        query         => $filter,
-        update        => $modifier,
-        %$options
+    # pass separately for MongoDB::Role::_BypassValidation
+    my $bypass = delete $options->{bypassDocumentValidation};
+
+    my $op = MongoDB::Op::_FindAndUpdate->_new(
+        filter         => $filter,
+        modifier       => $modifier,
+        options        => $options,
+        bypassDocumentValidation => $bypass,
+        %{ $self->_op_args },
     );
 
-    return $self->_try_find_and_modify( \@command );
+    return $self->client->send_write_op($op);
 }
 
 # we have a private _run_command rather than using the 'database' attribute
@@ -1374,20 +1500,6 @@ sub _run_command {
     return $obj->output;
 }
 
-sub _try_find_and_modify {
-    my ($self, $command) = @_;
-    my $result;
-    try {
-        $result = $self->_run_command( $command );
-    }
-    catch {
-        die $_ unless $_ eq 'No matching object found';
-    };
-
-    return $result->{value} if $result;
-    return;
-}
-
 #--------------------------------------------------------------------------#
 # utility function
 #--------------------------------------------------------------------------#
@@ -1446,7 +1558,7 @@ sub batch_insert {
         documents     => $documents,
         write_concern => $self->_dynamic_write_concern($opts),
         check_keys    => 0,
-        ordered       => 1, 
+        ordered       => 1,
     );
 
     my $result = $self->client->send_write_op($op);
@@ -1561,20 +1673,21 @@ sub save {
 
 sub find_and_modify {
     my ( $self, $opts ) = @_;
+    $opts ||= {};
 
-    my $conn = $self->client;
-    my $db   = $self->database;
+    MongoDB::UsageError->throw("find_and_modify requires a 'query' option")
+        unless $opts->{query};
 
-    my $result;
-    try {
-        $result = $db->run_command( [ findAndModify => $self->name, %$opts ] )
-    }
-    catch {
-        die $_ unless $_ eq 'No matching object found';
-    };
+    MongoDB::UsageError->throw("find_and_modify requires a 'remove' or 'update' option")
+        unless $opts->{remove} || $opts->{update};
 
-    return $result->{value} if $result;
-    return;
+    my $query = delete $opts->{query};
+    my $remove = delete $opts->{remove};
+    my $update = delete $opts->{update};
+
+    return $remove
+        ? $self->find_one_and_delete($query, $opts)
+        : $self->find_one_and_update($query, $update, $opts);
 }
 
 sub get_collection {
@@ -1716,7 +1829,7 @@ MongoDB::Collection - A MongoDB Collection
 
 =head1 VERSION
 
-version v1.0.4
+version v1.2.0
 
 =head1 SYNOPSIS
 
@@ -1828,6 +1941,14 @@ A L<MongoDB::WriteConcern> object.  It may be initialized with a hash
 reference that will be coerced into a new MongoDB::WriteConcern object.
 By default it will be inherited from a L<MongoDB::Database> object.
 
+=head2 read_concern
+
+A L<MongoDB::ReadConcern> object.  May be initialized with a hash
+reference or a string that will be coerced into the level of read
+concern.
+
+By default it will be inherited from a L<MongoDB::Database> object.
+
 =head2 max_time_ms
 
 Specifies the default maximum amount of time in milliseconds that the
@@ -1898,6 +2019,7 @@ to:
 =head2 insert_one
 
     $res = $coll->insert_one( $document );
+    $res = $coll->insert_one( $document, $options );
     $id = $res->inserted_id;
 
 Inserts a single L<document|/Document> into the database and returns a
@@ -1907,6 +2029,18 @@ If no C<_id> field is present, one will be added when a document is
 serialized for the database without modifying the original document.
 The generated C<_id> may be retrieved from the result object.
 
+An optional hash reference of options may be given.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=back
+
 =head2 insert_many
 
     $res = $coll->insert_many( [ @documents ] );
@@ -1921,11 +2055,21 @@ If no C<_id> field is present, one will be added when a document is
 serialized for the database without modifying the original document.
 The generated C<_id> may be retrieved from the result object.
 
-An optional hash reference of options may be provided.  The only valid option
-is C<ordered>, which defaults to true.  When true, the server will halt
-insertions after the first error (if any).  When false, all documents will be
-processed and any error will only be thrown after all insertions are
-attempted.
+An optional hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
+C<ordered> – when true, the server will halt insertions after the first error (if any).  When false, all documents will be processed and any error will only be thrown after all insertions are attempted.  The default is true.
+
+=back
 
 On MongoDB servers before version 2.6, C<insert_many> bulk operations are
 emulated with individual inserts to capture error information.  On 2.6 or
@@ -1961,9 +2105,21 @@ L<MongoDB::UnacknowledgedResult> object.
 The replacement document must not have any field-update operators in it (e.g.
 C<$set>).
 
-A hash reference of options may be provided.  The only valid option is
-C<upsert>, which defaults to false.  If provided and true, the replacement
-document will be upserted if no matching document exists.
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
+C<upsert> – defaults to false; if true, a new document will be added if one is not found
+
+=back
 
 =head2 update_one
 
@@ -1977,10 +2133,21 @@ object.
 The update document must have only field-update operators in it (e.g.
 C<$set>).
 
-A hash reference of options may be provided.  The only valid option is
-C<upsert>, which defaults to false.  If provided and true, a new document will
-be inserted by taking the filter expression and applying the update document
-operations to it prior to insertion.
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
+C<upsert> – defaults to false; if true, a new document will be added if one is not found by taking the filter expression and applying the update document operations to it prior to insertion.
+
+=back
 
 =head2 update_many
 
@@ -1994,10 +2161,21 @@ L<MongoDB::UnacknowledgedResult> object.
 The update document must have only field-update operators in it (e.g.
 C<$set>).
 
-A hash reference of options may be provided.  The only valid option is
-C<upsert>, which defaults to false.  If provided and true, a new document will
-be inserted by taking the filter document and applying the update document
-operations to it prior to insertion.
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
+C<upsert> – defaults to false; if true, a new document will be added if one is not found by taking the filter expression and applying the update document operations to it prior to insertion.
+
+=back
 
 =head2 find
 
@@ -2038,6 +2216,10 @@ C<limit> – the maximum number of documents to return.
 
 =item *
 
+C<maxAwaitTimeMS> – the maximum amount of time for the server to wait on new documents to satisfy a tailable cursor query. This only applies to a C<cursorType> of 'tailable_await'; the option is otherwise ignored. (Note, this will be ignored for servers before version 3.2.)
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time to allow the query to run. If C<$maxTimeMS> also exists in the modifiers document, the C<maxTimeMS> field overwrites C<$maxTimeMS>. (Note, this will be ignored for servers before version 2.6.)
 
 =item *
@@ -2115,6 +2297,20 @@ C<sort> – an L<ordered document|/Ordered document> defining the order in which
 See also core documentation on querying:
 L<http://docs.mongodb.org/manual/core/read/>.
 
+=head2 find_id
+
+    $doc = $collection->find_id( $id );
+    $doc = $collection->find_id( $id, $projection );
+    $doc = $collection->find_id( $id, $projection, $options );
+
+Executes a query with a L<filter expression|/Filter expression> of C<< { _id
+=> $id } >> and returns a single document.
+
+See the L<find_one|/find_one> documentation for details on the $projection and $options parameters.
+
+See also core documentation on querying:
+L<http://docs.mongodb.org/manual/core/read/>.
+
 =head2 find_one_and_delete
 
     $doc = $coll->find_one_and_delete( $filter );
@@ -2159,6 +2355,10 @@ A hash reference of options may be provided. Valid keys include:
 
 =item *
 
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.
 
 =item *
@@ -2196,6 +2396,10 @@ A hash reference of options may be provided. Valid keys include:
 
 =item *
 
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
 C<maxTimeMS> – the maximum amount of time in milliseconds to allow the command to run.  (Note, this will be ignored for servers before version 2.6.)
 
 =item *
@@ -2247,6 +2451,10 @@ C<batchSize> – the number of documents to return per batch.
 
 =item *
 
+C<bypassDocumentValidation> - skips document validation, if enabled. (Note, this will be ignored for servers before version 3.2.)
+
+=item *
+
 C<explain> – if true, return a single document with execution information.
 
 =item *
@@ -2375,6 +2583,18 @@ the first error. See L<MongoDB::BulkWrite> for more details.
 
 The method C<initialize_ordered_bulk_op> may be used as an alias.
 
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=back
+
 =head2 unordered_bulk
 
 This method works just like L</ordered_bulk> except that the order that
@@ -2383,6 +2603,18 @@ See L<MongoDB::BulkWrite> for more details.
 
 The method C<initialize_unordered_bulk_op> may be used as an alias.
 
+A hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=back
+
 =head2 bulk_write
 
     $res = $coll->bulk_write( [ @requests ], $options )
@@ -2426,10 +2658,21 @@ references:
 Valid method names include C<insert_one>, C<insert_many>, C<delete_one>,
 C<delete_many> C<replace_one>, C<update_one>, C<update_many>.
 
-An optional hash reference of options may be provided.  The only valid value
-is C<ordered>. It defaults to true.  When true, the bulk operation is executed
-like L</initialize_ordered_bulk>. When false, the bulk operation is executed
-like L</initialize_unordered_bulk>.
+An optional hash reference of options may be provided.
+
+Valid options include:
+
+=over 4
+
+=item *
+
+C<bypassDocumentValidation> - skips document validation, if enabled; this is ignored for MongoDB servers older than version 3.2.
+
+=item *
+
+C<ordered> – when true, the bulk operation is executed like L</initialize_ordered_bulk>. When false, the bulk operation is executed like L</initialize_unordered_bulk>.  The default is true.
+
+=back
 
 See L<MongoDB::BulkWrite> for more details on bulk writes.  Be advised that
 the legacy Bulk API method names differ slightly from MongoDB::Collection
