@@ -20,7 +20,7 @@ package MongoDB::Cursor;
 # ABSTRACT: A lazy cursor for Mongo query results
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use Moo;
 use MongoDB;
@@ -29,6 +29,7 @@ use MongoDB::Error;
 use MongoDB::QueryResult;
 use MongoDB::ReadPreference;
 use MongoDB::_Protocol;
+use MongoDB::Op::_Explain;
 use MongoDB::_Types -types, 'to_IxHash';
 use Types::Standard qw(
     InstanceOf
@@ -182,6 +183,31 @@ sub limit {
 }
 
 
+#pod =head2 max_await_time_ms
+#pod
+#pod     $cursor->max_await_time_ms( 500 );
+#pod
+#pod The maximum amount of time in milliseconds for the server to wait on new
+#pod documents to satisfy a tailable cursor query. This only applies to a
+#pod cursor of type 'tailble_await'.  This is ignored if the cursor is not
+#pod a 'tailable_await' cursor or the server version is less than version 3.2.
+#pod
+#pod Returns this cursor for chaining operations.
+#pod
+#pod =cut
+
+sub max_await_time_ms {
+    my ( $self, $num ) = @_;
+    $num = 0 unless defined $num;
+    MongoDB::UsageError->throw("max_await_time_ms must be non-negative")
+      if $num < 0;
+    MongoDB::UsageError->throw("can not set max_await_time_ms after querying")
+      if $self->started_iterating;
+
+    $self->query->maxAwaitTimeMS( $num );
+    return $self;
+}
+
 #pod =head2 max_time_ms
 #pod
 #pod     $cursor->max_time_ms( 500 );
@@ -421,15 +447,16 @@ sub read_preference {
 sub explain {
     my ($self) = @_;
 
-    my $new_query = $self->query->clone;
-    $new_query->modifiers->{'$explain'} = true;
+    my $explain_op = MongoDB::Op::_Explain->_new(
+        db_name         => $self->query->db_name,
+        coll_name       => $self->query->coll_name,
+        bson_codec      => $self->query->bson_codec,
+        query           => $self->query->clone,
+        read_preference => $self->query->read_preference,
+        read_concern    => $self->query->read_concern,
+    );
 
-    # per David Storch, drivers *must* send a negative limit to instruct
-    # the query planner analysis module to add a LIMIT stage.  For older
-    # explain implementations, it also ensures a cursor isn't left open.
-    $new_query->limit( -1 * abs( $new_query->limit ) );
-
-    return $new_query->execute->next;
+    return $self->query->client->send_read_op($explain_op);
 }
 
 #pod =head1 QUERY ITERATION
@@ -483,6 +510,19 @@ sub has_next { $_[0]->result->has_next }
 
 sub next { $_[0]->result->next }
 
+#pod =head2 batch
+#pod
+#pod     while (my @batch = $cursor->batch) {
+#pod         ...
+#pod     }
+#pod
+#pod Returns the next batch of data from the cursor. Will automatically fetch more
+#pod data from the server if necessary. Returns an empty list if no more data is available.
+#pod
+#pod =cut
+
+sub batch { $_[0]->result->batch }
+
 #pod =head2 all
 #pod
 #pod     my @objects = $cursor->all;
@@ -514,8 +554,7 @@ sub reset {
 #pod production use.  Currently the fields are:
 #pod
 #pod =for :list
-#pod * C<cursor_id>  -- the server-side id for this cursor as.  This is an opaque string.
-#pod   A C<cursor_id> of "\0\0\0\0\0\0\0\0" means there are no more results on the server.
+#pod * C<cursor_id>  -- the server-side id for this cursor.  See below for details.
 #pod * C<num> -- the number of results received from the server so far
 #pod * C<at> -- the (zero-based) index of the document that will be returned next from L</next>
 #pod * C<flag> -- if the database could not find the cursor or another error occurred, C<flag> may
@@ -527,6 +566,15 @@ sub reset {
 #pod If the cursor has not yet executed, only the C<num> field will be returned with
 #pod a value of 0.
 #pod
+#pod The C<cursor_id> could appear in one of three forms:
+#pod
+#pod =for :list
+#pod * MongoDB::CursorID object (a blessed reference to an 8-byte string)
+#pod * A perl scalar (an integer)
+#pod * A Math::BigInt object (64 bit integer on 32-bit perl)
+#pod
+#pod When the C<cursor_id> is zero, there are no more results to fetch.
+#pod
 #pod =cut
 
 sub info {
@@ -614,7 +662,7 @@ MongoDB::Cursor - A lazy cursor for Mongo query results
 
 =head1 VERSION
 
-version v1.0.4
+version v1.2.0
 
 =head1 SYNOPSIS
 
@@ -701,6 +749,17 @@ Sets cursor to return a maximum of N results.
 
 Returns this cursor for chaining operations.
 
+=head2 max_await_time_ms
+
+    $cursor->max_await_time_ms( 500 );
+
+The maximum amount of time in milliseconds for the server to wait on new
+documents to satisfy a tailable cursor query. This only applies to a
+cursor of type 'tailble_await'.  This is ignored if the cursor is not
+a 'tailable_await' cursor or the server version is less than version 3.2.
+
+Returns this cursor for chaining operations.
+
 =head2 max_time_ms
 
     $cursor->max_time_ms( 500 );
@@ -851,6 +910,15 @@ data from the server if necessary.
 Returns the next object in the cursor. Will automatically fetch more data from
 the server if necessary. Returns undef if no more data is available.
 
+=head2 batch
+
+    while (my @batch = $cursor->batch) {
+        ...
+    }
+
+Returns the next batch of data from the cursor. Will automatically fetch more
+data from the server if necessary. Returns an empty list if no more data is available.
+
 =head2 all
 
     my @objects = $cursor->all;
@@ -873,7 +941,7 @@ production use.  Currently the fields are:
 
 =item *
 
-C<cursor_id>  -- the server-side id for this cursor as.  This is an opaque string. A C<cursor_id> of "\0\0\0\0\0\0\0\0" means there are no more results on the server.
+C<cursor_id>  -- the server-side id for this cursor.  See below for details.
 
 =item *
 
@@ -896,6 +964,26 @@ C<start> -- the index of the result that the current batch of results starts at.
 If the cursor has not yet executed, only the C<num> field will be returned with
 a value of 0.
 
+The C<cursor_id> could appear in one of three forms:
+
+=over 4
+
+=item *
+
+MongoDB::CursorID object (a blessed reference to an 8-byte string)
+
+=item *
+
+A perl scalar (an integer)
+
+=item *
+
+A Math::BigInt object (64 bit integer on 32-bit perl)
+
+=back
+
+When the C<cursor_id> is zero, there are no more results to fetch.
+
 =head1 SEE ALSO
 
 Core documentation on cursors: L<http://dochub.mongodb.org/core/cursors>.
