@@ -20,7 +20,7 @@ package MongoDB::Database;
 # ABSTRACT: A MongoDB Database
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use MongoDB::CommandResult;
 use MongoDB::Error;
@@ -32,6 +32,7 @@ use MongoDB::_Types qw(
     BSONCodec
     NonNegNum
     ReadPreference
+    ReadConcern
     WriteConcern
 );
 use Types::Standard qw(
@@ -44,7 +45,7 @@ use Moo;
 use Try::Tiny;
 use namespace::clean -except => 'meta';
 
-has _client => ( 
+has _client => (
     is       => 'ro',
     isa      => InstanceOf['MongoDB::MongoClient'],
     required => 1,
@@ -93,6 +94,23 @@ has write_concern => (
     coerce   => WriteConcern->coercion,
 );
 
+#pod =attr read_concern
+#pod
+#pod A L<MongoDB::ReadConcern> object.  May be initialized with a hash
+#pod reference or a string that will be coerced into the level of read
+#pod concern.
+#pod
+#pod By default it will be inherited from a L<MongoDB::MongoClient> object.
+#pod
+#pod =cut
+
+has read_concern => (
+    is       => 'ro',
+    isa      => ReadConcern,
+    required => 1,
+    coerce   => ReadConcern->coercion,
+);
+
 #pod =attr max_time_ms
 #pod
 #pod Specifies the maximum amount of time in milliseconds that the server should use
@@ -234,6 +252,7 @@ sub get_collection {
     return MongoDB::Collection->new(
         read_preference => $self->read_preference,
         write_concern   => $self->write_concern,
+        read_concern    => $self->read_concern,
         bson_codec      => $self->bson_codec,
         max_time_ms     => $self->max_time_ms,
         ( $options ? %$options : () ),
@@ -388,7 +407,7 @@ MongoDB::Database - A MongoDB Database
 
 =head1 VERSION
 
-version v1.0.4
+version v1.2.0
 
 =head1 SYNOPSIS
 
@@ -456,6 +475,14 @@ A L<MongoDB::WriteConcern> object.  It may be initialized with a hash
 reference that will be coerced into a new MongoDB::WriteConcern object.
 By default it will be inherited from a L<MongoDB::MongoClient> object.
 
+=head2 read_concern
+
+A L<MongoDB::ReadConcern> object.  May be initialized with a hash
+reference or a string that will be coerced into the level of read
+concern.
+
+By default it will be inherited from a L<MongoDB::MongoClient> object.
+
 =head2 max_time_ms
 
 Specifies the maximum amount of time in milliseconds that the server should use
