@@ -19,7 +19,7 @@ package MongoDB::MongoClient;
 # ABSTRACT: A connection to a MongoDB server or multi-server deployment
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use Moo;
 use MongoDB;
@@ -30,8 +30,10 @@ use MongoDB::Cursor;
 use MongoDB::DBRef;
 use MongoDB::Error;
 use MongoDB::Op::_Command;
+use MongoDB::Op::_FSyncUnlock;
 use MongoDB::ReadPreference;
 use MongoDB::WriteConcern;
+use MongoDB::ReadConcern;
 use MongoDB::_Topology;
 use MongoDB::_Constants;
 use MongoDB::_Credential;
@@ -57,9 +59,11 @@ use Types::Standard qw(
     Bool
     HashRef
     InstanceOf
+    Undef
     Int
     Num
     Str
+    Maybe
 );
 
 use namespace::clean -except => 'meta';
@@ -779,7 +783,7 @@ sub _build_username {
 
 has w => (
     is      => 'lazy',
-    isa     => Int|Str,
+    isa     => Int|Str|Undef,
     builder => '_build_w',
 );
 
@@ -788,7 +792,7 @@ sub _build_w {
     return $self->__uri_or_else(
         u => 'w',
         e => 'w',
-        d => 1,
+        d => undef,
     );
 }
 
@@ -820,6 +824,40 @@ sub _build_wtimeout {
     );
 }
 
+#pod =attr read_concern_level
+#pod
+#pod The read concern level determines the consistency level required
+#pod of data being read.
+#pod
+#pod The default level is C<undef>, which means the server will use its configured
+#pod default.
+#pod
+#pod If the level is set to "local", reads will return the latest data a server has
+#pod locally.
+#pod
+#pod Additional levels are storage engine specific.  See L<Read
+#pod Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
+#pod documentation for more details.
+#pod
+#pod This may be set in a connection string with the the C<readConcernLevel> option.
+#pod
+#pod =cut
+
+has read_concern_level => (
+    is      => 'lazy',
+    isa     => Maybe [Str],
+    builder => '_build_read_concern_level',
+);
+
+sub _build_read_concern_level {
+    my ($self) = @_;
+    return $self->__uri_or_else(
+        u => 'readconcernlevel',
+        e => 'read_concern_level',
+        d => undef,
+    );
+}
+
 #--------------------------------------------------------------------------#
 # deprecated public attributes
 #--------------------------------------------------------------------------#
@@ -962,6 +1000,30 @@ sub _build__write_concern {
     );
 }
 
+#pod =method read_concern
+#pod
+#pod Returns a L<MongoDB::ReadConcern> object constructed from
+#pod L</read_concern_level>.
+#pod
+#pod =cut
+
+has _read_concern => (
+    is     => 'lazy',
+    isa    => InstanceOf['MongoDB::ReadConcern'],
+    reader   => 'read_concern',
+    init_arg => undef,
+    builder  => '_build__read_concern',
+);
+
+sub _build__read_concern {
+    my ($self) = @_;
+
+    return MongoDB::ReadConcern->new(
+        ( $self->read_concern_level ?
+            ( level => $self->read_concern_level ) : () ),
+    );
+}
+
 #--------------------------------------------------------------------------#
 # private attributes
 #--------------------------------------------------------------------------#
@@ -1096,6 +1158,7 @@ my @deferred_options = qw(
   password
   w
   wtimeout
+  read_concern_level
 );
 
 around BUILDARGS => sub {
@@ -1414,6 +1477,7 @@ sub get_database {
     return MongoDB::Database->new(
         read_preference => $self->read_preference,
         write_concern   => $self->write_concern,
+        read_concern    => $self->read_concern,
         bson_codec      => $self->bson_codec,
         max_time_ms     => $self->max_time_ms,
         ( $options ? %$options : () ),
@@ -1494,8 +1558,13 @@ sub fsync {
 sub fsync_unlock {
     my ($self) = @_;
 
-    # Have to fetch from a special collection to unlock.
-    return $self->get_database('admin')->get_collection('$cmd.sys.unlock')->find_one();
+    my $op = MongoDB::Op::_FSyncUnlock->_new(
+        db_name    => 'admin',
+        client     => $self,
+        bson_codec => $self->bson_codec,
+    );
+
+    return $self->send_read_op($op);
 }
 
 
@@ -1513,7 +1582,7 @@ MongoDB::MongoClient - A connection to a MongoDB server or multi-server deployme
 
 =head1 VERSION
 
-version v1.0.4
+version v1.2.0
 
 =head1 SYNOPSIS
 
@@ -1976,6 +2045,23 @@ See C<w> above for more information.
 
 This may be set in a connection string with the C<wTimeoutMS> option.
 
+=head2 read_concern_level
+
+The read concern level determines the consistency level required
+of data being read.
+
+The default level is C<undef>, which means the server will use its configured
+default.
+
+If the level is set to "local", reads will return the latest data a server has
+locally.
+
+Additional levels are storage engine specific.  See L<Read
+Concern|http://docs.mongodb.org/manual/search/?query=readConcern> in the MongoDB
+documentation for more details.
+
+This may be set in a connection string with the the C<readConcernLevel> option.
+
 =head2 dt_type (DEPRECATED AND READ-ONLY)
 
 Sets the type of object which is returned for DateTime fields. The default
@@ -2035,6 +2121,11 @@ C<get_collection>.
 Returns a L<MongoDB::WriteConcern> object constructed from L</w>, L</write_concern>
 and L</j>.
 
+=head2 read_concern
+
+Returns a L<MongoDB::ReadConcern> object constructed from
+L</read_concern_level>.
+
 =head2 topology_type
 
 Returns an enumerated topology type.  If the L</replica_set_name> is
