@@ -20,7 +20,7 @@ package MongoDB::Op::_BulkWrite;
 # MongoDB::BulkWriteResult object
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use Moo;
 
@@ -84,6 +84,7 @@ with $_ for qw(
   MongoDB::Role::_CommandOp
   MongoDB::Role::_UpdatePreEncoder
   MongoDB::Role::_InsertPreEncoder
+  MongoDB::Role::_BypassValidation
 );
 
 sub execute {
@@ -193,9 +194,13 @@ sub _execute_write_command_batch {
             $cmd         => $coll_name,
             $op_key      => $chunk,
             ordered      => $boolean_ordered,
-            writeConcern => $wc->as_struct,
+            @{ $wc->as_args },
         ];
 
+        if ( $cmd eq 'insert' || $cmd eq 'update' ) {
+            (undef, $cmd_doc) = $self->_maybe_bypass($link, $cmd_doc);
+        }
+
         my $op = MongoDB::Op::_Command->_new(
             db_name     => $db_name,
             query       => $cmd_doc,
@@ -319,7 +324,8 @@ sub _execute_legacy_batch {
     my $wc  = $self->write_concern;
     my $w_0 = !$wc->is_acknowledged;
     if ($w_0) {
-        my $wcs = $wc->as_struct;
+        my $wc_args = $wc->as_args();
+        my $wcs = scalar @$wc_args ? $wc->as_args()->[1] : {};
         $wcs->{w} = 1;
         $wc = MongoDB::WriteConcern->new($wcs);
     }
