@@ -19,7 +19,7 @@ package MongoDB::Op::_KillCursors;
 # Encapsulate a cursor kill operation; returns true
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use Moo;
 
@@ -34,7 +34,7 @@ use namespace::clean;
 has cursor_ids => (
     is       => 'ro',
     required => 1,
-    isa      => ArrayRef [Str],
+    isa      => ArrayRef,
 );
 
 with $_ for qw(
