@@ -19,7 +19,7 @@ package MongoDB::QueryResult;
 # ABSTRACT: An iterator for Mongo query results
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use Moo;
 use MongoDB::Error;
@@ -31,7 +31,9 @@ use MongoDB::_Types qw(
     HostAddress
 );
 use Types::Standard qw(
+    Maybe
     ArrayRef
+    Any
     InstanceOf
     Int
     HashRef
@@ -77,6 +79,11 @@ has _batch_size => (
     isa      => Int,
 );
 
+has _max_time_ms => (
+    is       => 'ro',
+    isa      => Maybe[Num],
+);
+
 # attributes for tracking progress
 
 has _cursor_at => (
@@ -95,11 +102,12 @@ has _limit => (
 
 # attributes from actual results
 
+# integer or MongoDB::_CursorID or Math::BigInt
 has _cursor_id => (
     is       => 'ro',
     required => 1,
     writer   => '_set_cursor_id',
-    isa => Str,
+    isa => Any,
 );
 
 has _cursor_start => (
@@ -197,19 +205,42 @@ sub next {
     return $self->_next_doc;
 }
 
+#pod =method batch
+#pod
+#pod   while ( @batch = $result->batch ) {
+#pod     for $doc ( @batch ) {
+#pod       process_doc($doc);
+#pod     }
+#pod   }
+#pod
+#pod Returns the next batch of documents or an empty list if the server cursor is exhausted.
+#pod
+#pod =cut
+
+sub batch {
+  my ($self) = @_;
+  return unless $self->has_next;
+  return $self->_drain_docs;
+}
+
 sub _get_more {
     my ($self) = @_;
-    return 0 if $self->_cursor_id eq CURSOR_ZERO;
+    return 0 if $self->_cursor_id == 0;
 
     my $limit = $self->_limit;
     my $want = $limit > 0 ? ( $limit - $self->_cursor_at ) : $self->_batch_size;
 
+    my ($db_name, $coll_name) = split(/\./, $self->_ns, 2);
+
     my $op = MongoDB::Op::_GetMore->_new(
-        ns         => $self->_ns,
-        client     => $self->_client,
-        bson_codec => $self->_bson_codec,
-        cursor_id  => $self->_cursor_id,
-        batch_size => $want,
+        ns          => $self->_ns,
+        db_name     => $db_name,
+        coll_name   => $coll_name,
+        client      => $self->_client,
+        bson_codec  => $self->_bson_codec,
+        cursor_id   => $self->_cursor_id,
+        batch_size  => $want,
+        max_time_ms => $self->_max_time_ms,
     );
 
     my $result = $self->_client->send_direct_op( $op, $self->_address );
@@ -239,10 +270,11 @@ sub all {
 
 sub _kill_cursor {
     my ($self) = @_;
-    return if !defined $self->_cursor_id || $self->_cursor_id eq CURSOR_ZERO;
-    my $op = MongoDB::Op::_KillCursors->_new( cursor_ids => [ $self->_cursor_id ], );
+    my $cursor_id = $self->_cursor_id;
+    return if !defined $cursor_id || $cursor_id == 0;
+    my $op = MongoDB::Op::_KillCursors->_new( cursor_ids => [ $cursor_id ], );
     $self->_client->send_direct_op( $op, $self->_address );
-    $self->_set_cursor_id(CURSOR_ZERO);
+    $self->_set_cursor_id(0);
 }
 
 sub DEMOLISH {
@@ -250,38 +282,6 @@ sub DEMOLISH {
     $self->_kill_cursor;
 }
 
-#--------------------------------------------------------------------------#
-# utility functions
-#--------------------------------------------------------------------------#
-
-# If we get a cursor_id from a command, BSON decoding will give us either
-# a perl scalar or a Math::BigInt object (if we don't have 32 bit support).
-# For OP_GET_MORE, we treat it as an opaque string, so we need to convert back
-# to a packed, little-endian quad
-sub _pack_cursor_id {
-    my $cursor_id = shift;
-    if ( ref($cursor_id) eq "Math::BigInt" ) {
-        my $as_hex = $cursor_id->as_hex; # big-endian hex
-        substr( $as_hex, 0, 2, '' );     # remove "0x"
-        my $len = length($as_hex);
-        substr( $as_hex, 0, 0, "0" x ( 16 - $len ) ) if $len < 16; # pad to quad length
-        $cursor_id = pack( "H*", $as_hex );                        # packed big-endian
-        $cursor_id = reverse($cursor_id);                          # reverse to little-endian
-    }
-    elsif (HAS_INT64) {
-        # pack doesn't have endianness modifiers before perl 5.10.
-        # We die during configuration on big-endian platforms on 5.8
-        $cursor_id = pack( $] lt '5.010' ? "q" : "q<", $cursor_id );
-    }
-    else {
-        # we on 32-bit perl *and* have a cursor ID that fits in 32 bits,
-        # so pack it as long and pad out to a quad
-        $cursor_id = pack( $] lt '5.010' ? "l" : "l<", $cursor_id ) . ( "\0" x 4 );
-    }
-
-    return $cursor_id;
-}
-
 #pod =head1 SYNOPSIS
 #pod
 #pod     $cursor = $coll->find( $filter );
@@ -335,7 +335,7 @@ MongoDB::QueryResult - An iterator for Mongo query results
 
 =head1 VERSION
 
-version v1.0.4
+version v1.2.0
 
 =head1 SYNOPSIS
 
@@ -394,6 +394,16 @@ necessary.
 
 Returns the next document or C<undef> if the server cursor is exhausted.
 
+=head2 batch
+
+  while ( @batch = $result->batch ) {
+    for $doc ( @batch ) {
+      process_doc($doc);
+    }
+  }
+
+Returns the next batch of documents or an empty list if the server cursor is exhausted.
+
 =head2 all
 
     @docs = $result->all;
