@@ -19,7 +19,7 @@ package MongoDB::Role::_CommandOp;
 # MongoDB interface for database command operations
 
 use version;
-our $VERSION = 'v1.0.4';
+our $VERSION = 'v1.2.0';
 
 use MongoDB::BSON;
 use MongoDB::Error;
@@ -30,7 +30,7 @@ use namespace::clean;
 
 with 'MongoDB::Role::_DatabaseOp';
 
-requires qw/db_name/;
+requires qw/db_name bson_codec/;
 
 sub _send_command {
     my ( $self, $link, $doc, $flags ) = @_;
